import type { PackageJson } from 'type-fest';
import WebpackDevServer from 'webpack-dev-server';
import type { PostCssOptions, RemConfig } from './common/postcssConfig';
import webpack, { Configuration, StatsOptions } from 'webpack';
import type { Options as LodashModuleReplacementPluginOptions } from 'lodash-webpack-plugin';
import autoprefixer from 'autoprefixer';
import type { TerserOptions } from 'terser-webpack-plugin';
import type { MinimizerOptions } from 'css-minimizer-webpack-plugin';
import type { PostcssLess2varOptions } from './common/postcss-plugin-less2var';
import type { VueLoaderOptions } from 'vue-loader';
import type { AntDesignVueResolverOptions } from 'unplugin-vue-components/resolvers';
import { BundleStatsWebpackPlugin } from 'bundle-stats-webpack-plugin';
type Output = any;
type Devtool = false | 'eval' | string;
export type Readonly<T> = {
    readonly [P in keyof T]: T[P];
};
export type Nullable<T> = {
    [P in keyof T]: T[P] | null;
};
export interface Dictionary<T> {
    [key: string]: T;
}
export interface NumericDictionary<T> {
    [key: number]: T;
}
export type IBabelPresetOrPlugin = string | [string, any, string?];
export type IPackage = PackageJson;
export interface IImportPluginOpts {
    libraryName: string;
    libraryDirectory?: string;
    style?: boolean;
    camel2DashComponentName?: boolean;
}
export interface IBabelPresetOpts {
    nodeEnv: Mode;
    env: {
        targets?: IBabelTargets;
    };
    import: IImportPluginOpts[];
    lodash?: false | {
        id?: string | string[];
        cwd?: string;
    };
}
export interface IBabelOptions {
    cwd: string;
    presetOpts: IBabelPresetOpts;
}
export interface IBaseUserConfig {
    /**
     * 国际化标记函数 - 建议不要修改
     * @default "languageChange"
     */
    languageWrapper?: string;
    /**
     * 使用typescript 开发
     */
    typescript?: boolean;
    /**
     * 指定读取配置
     */
    scene?: string | false;
    /**
     * dev server port
     */
    port?: number;
    /**
     * dev server port
     */
    cors?: {
        origin?: string;
        credentials?: boolean;
        methods?: string[];
        headers?: string[];
        expose?: string[];
    };
    client?: WebpackDevServer.ClientConfiguration;
    /**
     * proxy config
     */
    before?: WebpackDevServer.Configuration['setupMiddlewares'];
    proxy?: WebpackDevServer.ProxyConfigMap;
    /**
     * 是否打印日志
     * @default false
     */
    log?: boolean;
    /**
     * less 文件处理方式，缺省时会使用 webpack 处理 less 以及其他静态资源，开启后，会单独处理 less 文件
     *
     * @default false
     */
    useLess?: boolean;
    /**
     * 当 useLess 为 true 时，less --> less or css，缺省时转为 css
     *
     * @default false
     */
    toLess?: boolean;
    /**
     * 当 toLess 为 true 时，输出css文件
     *
     * @default false
     */
    withCss?: boolean;
    /**
     * 当压缩输出css文件
     *
     * @default false
     */
    compressStyle?: boolean;
    /**
     * 针对 webpack（js） 的 entry 配置
     */
    entry?: Dictionary<string>;
    /**
     * 针对 less 的 entry 配置
     */
    entryStyle?: string[];
    /**
     * 产物路径，支持相对路径 or 绝对路径
     */
    dist?: string;
    /**
     * docker 位置
     */
    docker?: string;
    /**
     * js 文件扫描的基本路径
     *
     * @default './src'
     */
    base?: string;
    scan?: string[] | {
        asset?: string;
        js?: string;
    };
    /**
     * less 文件扫描的基本路径
     *
     * @default './src'
     */
    baseLess?: string;
    scanLess?: string[];
    /**
     * 是否开启 webpack 的性能监测
     *
     * @default false
     */
    profile?: boolean;
    /**
     * 是否开启打包过程中的语法检查
     *
     * @default true
     */
    eslint?: boolean;
    /**
     * 是否开启打包过程中的样式语法检查
     *
     * @default true
     */
    stylelint?: boolean;
    /**
     * source map 类型
     *
     * @default 'source-map'
     */
    devtool?: Devtool;
    /**
     * webpack 的 output 配置
     */
    output?: Output;
    /**
     * webpack 的 externals 配置
     */
    externals?: Dictionary<string>;
    preCommit?: {
        eslint?: boolean;
        prettier?: boolean;
    };
    /**
     * babel 的 targets 配置
     *
     * @default { chrome: 49, firefox: 64, safari: 10, edge: 13, ios: 10 }
     */
    targets?: IBabelTargets;
    babelImport?: IImportPluginOpts[];
    useRem?: boolean;
    remConfig?: Partial<RemConfig>;
    themeMappings?: Record<string, string | ((importPath: string) => string)>;
    /**
     * 是否清除输出目录
     */
    clean?: boolean;
    cleanPatterns?: 'default' | 'auto' | string[];
    stats?: StatsOptions;
    logging?: Configuration['infrastructureLogging'];
    lodashOptions?: boolean | (LodashModuleReplacementPluginOptions & {
        babel?: IBabelPresetOpts['lodash'];
    });
    /**
     * 运行时动态path
     */
    dynamicPublicPath?: boolean;
    dynamicCssUrlOptions?: PostCssOptions['dynamicCssUrlConfig'];
    /**
     * less变量
     */
    modifyLessVars?: Record<string, any>;
    /**
     * 前缀替换配置
     */
    prefixConfig?: Record<string, string>;
    less2CssVarConfig?: PostcssLess2varOptions & {
        enabled: boolean;
    };
    modifyPrefix?: {
        prefix?: Record<string, string>;
        enabled?: boolean;
        excludes?: RegExp[];
        includes?: RegExp[];
        test?: (file: string) => boolean;
    };
    autoPrefixerOptions?: autoprefixer.Options;
    miniOptions?: {
        terser?: TerserOptions;
        /**
         * @default true
         */
        compressCss?: boolean;
        css?: MinimizerOptions<any>;
    };
    scanFiles?: string[];
    webpackPlugins?: webpack.Configuration['plugins'];
    noDojo?: boolean;
    pack?: {
        outDir?: string;
        file?: string;
        excludeDirs?: string[];
        excludes?: string[];
        verbose?: boolean;
    };
    transCompile?: webpack.RuleSetRule['include'];
    vueLoaderOptions?: VueLoaderOptions;
    antdVResolverOptions?: AntDesignVueResolverOptions;
    optimization?: webpack.Configuration['optimization'];
    reportProfile?: {
        console?: boolean;
        file?: boolean;
    };
    bundleStats?: BundleStatsWebpackPlugin.Options;
    webpack?: (config: webpack.Configuration) => void | webpack.Configuration;
    alias?: webpack.Configuration['resolve']['alias'];
}
export type IBabelTargets = Dictionary<string | number> | {
    node: number | string | 'current|true';
} | {
    browsers: string[] | string;
};
export interface IUserConfig extends IBaseUserConfig {
    dev?: IBaseUserConfig;
}
export type Mode = 'development' | 'production';
export interface IBundleOptions extends IBaseUserConfig {
    base: string;
    baseLess: string;
    /**
     * 绝对路径
     */
    absDir: string;
    /**
     * 当前根目录
     */
    cwd: string;
    layout: boolean;
    projectName: string;
    mode: Mode;
    /**
     * 是否启用基于 router 索引进行打包
     * @default false
     */
    routerIndex?: boolean;
    /**
     * 是否开启 react props 自动移除（一般用于生产模式）
     * @default false
     */
    reactRemovePropTypes?: boolean;
    pkg: IPackage;
    devtool: Devtool;
}
export {};
